/*
 * Опросы:
 */
// Load all polls instead of their placeholders:
$(function () {
	// Polls:
	$ ('form.poll').each (function () {
		loadPoll (this);
	});
});

// Optionally send; then load a poll:
function loadPoll (/* DOM */ form, /* string */ chosenID) {
	var $form = $(form);
	var id = $form.attr ('id').substr (5);
	if (id) {
		var api_params = {
			'action': 'poll'
		  , 'format': 'json'
		  , 'poll'  : id
		};
		if (chosenID) {
			api_params.option = chosenID;
		}
		// Call MediaWiki API:
		$.ajax (poll_api_path, {
			dataType: 'json'
		  , cache   : false
		  , data    : api_params
		  , success : function (data) {
				if (data && data.options && data.options.length) {
					var total_votes = data.pollvotes && $.isNumeric (data.pollvotes) ? parseInt (data.pollvotes) : 0;
					var show_votes = data.uservote || !data.canvote || poll_show;
					// Answer options:
					var options = $.map (data.options, function (option, order) {
						var text    = option.optiontext;
						var votes   = $.isNumeric (option.optionvotes) ? parseInt (option.optionvotes) : 0;
						var percent = total_votes !== 0 ? (votes / total_votes * 100).toFixed (1) + '%' : '';
						var gradient_percent = total_votes !== 0 && show_votes ? Math.floor (votes / total_votes * 100) + '%' : '100%';
						var color = mw.message ('poll-option-color-' + (order % data.options.length));
						var gradient = 'linear-gradient(to right, #FFFFFF 0%, '
									 + color + ' '
									 + gradient_percent + ', #FFFFFF ' + gradient_percent + ')';
						var checked = option.optionid === data.uservote ? 'checked' : '';
						var msgid   = data.canvote	? show_votes ? 'poll-option' : 'poll-option-nostats'
													: 'poll-option-readonly';
						return $(mw.message (msgid, id, option.optionid, checked, text, votes, percent).text ()).css ('background-image', gradient);
					}); // -- var options = ...
					$form.find ('.options').empty ().append (options);
					// Totals:
					if (data.pollvotes && show_votes) {
						$form.find ('.total').html (data.pollvotes);
					}
					// Submit button:
					if (data.canvote) {
						var $button;
						if (data.uservote) {
							if (poll_revote) {
								$button = $('<button type="submit">' + mw.message ('poll-resubmit').plain () + '</button>');
							} else {
								$button = $(mw.message ('poll-voted').plain ());
							}
						} else {
							$button = $('<button type="submit">' + mw.message ('poll-submit').plain () + '</button>');
						} // -- if (data.uservote)
						$form.find ('.button').empty ().append ($button);
						$form.submit (sendPoll);
					}
				} // -- if (data && data.options && data.options.length)
			} // -- success: function (data)
		}); // -- $.ajax ({...})
	} // -- if (id)
} // -- function loadPoll (/* DOM */ form)

function sendPoll () {
	var $this = $(this);
	$this.find ('.button').html (mw.message ('poll-sent').plain ());
	loadPoll (this, $this.find ('input:checked').val ());
	return false;
}

/*
 * Рейтинги страниц:
 */
// Add ratings widgets:
$(function () {
	$ ('div.ratingWidget').each (function () {
		loadRating (this);
	});
});

// Optionally send; then load a poll:
function loadRating (/* DOM */ div, /* integer */ chosenRating) {
	//window.alert ('In loadRating!');
	var api_params = {
		'action': 'rate'
	  , 'format': 'json'
	  , 'pageid': mw.config.get ('wgArticleId')
	};
	if (chosenRating) {
		api_params.rating = chosenRating;
	}
	$.ajax ('/w/api.php', {
		dataType: 'json'
	  , cache   : false
	  , data    : api_params
	  , success : function (data) {
			if (data) {
				var page_votes  = $.isNumeric (data.pagevotes)  ? parseInt (data.pagevotes)  : 0;
				var page_rating = $.isNumeric (data.pagerating) ? parseInt (data.pagerating) : 0;
				var user_rating = $.isNumeric (data.userrating) ? parseInt (data.userrating) : 0;
				var rating = page_votes ? page_rating / page_votes : 0;
				var hex00 = function (no) {
					var unpadded = Math.floor (no).toString (16);
					return ('00' + unpadded).substr (- 2, 2).toUpperCase ();
				}
				var blue  = '#4060FF';
				var red   = '#F02040';
				var green = '#78B038';
				var color = rating
						  ? ('#'
						  + hex00 (135 * (5 - rating) / 4 + 120)
						  + hex00 (240 * (rating - 1) / 4 + 15)
						  + '30')
						  : red;
				var gradient_percent = Math.floor (rating / 5 * 100) + '%';
				var gradient = 'linear-gradient(to right, ' + red + ' 0%, '
							 + color + ' '
							 + gradient_percent + ', ' + blue + ' ' + gradient_percent + ', ' + blue + ' 100%)';
				var $rate_widget = $(mw.message ('poll-rating-widget', gradient).plain ());
				var verbal_chosen = mw.message ('poll-rating-' + user_rating);
				var raters = page_votes
						   ? mw.message ('poll-page-raters', rating.toFixed (1), data.pagevotes)
						   : (mw.message ('poll-page-no-raters')
						   + (data.canrate ? ' ' + mw.message ('poll-page-you-first') : ''));
				var your_rating = data.canrate
								? (user_rating === 0
								? (page_votes ? mw.message ('poll-rating-not-rated') : '')
								: mw.message ('poll-rating-rated', user_rating, verbal_chosen))
								: '';
				// A function that creates a function:
				var make_clicker = function (no) {
					return function () {
						loadRating (div, no);
					};
				};
				for (var i = 1; i <= 5; i++) {
					var verbal = mw.message ('poll-rating-' + i);
					var proposal = data.canrate
								 ? (page_rating === 0
								 ? mw.message ('poll-rating-rate', i, verbal) //mw.message ('poll-rating-rate-first')
								 : i === user_rating
								 ? mw.message ('poll-rating-rerate')
								 : mw.message ('poll-rating-rate', i, verbal))
								 : '';
					var title = raters + ' ' + your_rating + ' ' + proposal;
					var $figure = $('<span' + (i === user_rating ? ' class="selected"' : '') + '>' + i + '</span>').attr ('title', title);
					if (data.canrate) {
						$figure.click (make_clicker (i));
					}
					$rate_widget.append ($figure);
				} // -- for (var i = 1; i <= 5; i++)
				//window.alert (chosenRating);
				$(div).empty ().append ($rate_widget);
			} // -- if (data)
		} // -- success : function (data)
	}); // -- $.ajax ('/w/api.php', {...});
} // -- loadRating (/* DOM */ form, /* integer */ chosenRating)

// Отладочная функция:
function showObj (obj, depth) {
    var out = '';
    if (!depth) {
        depth = 2;
    }
    var level = '';
	var type = $.type (obj);
	if (type === 'array' && obj !== null && depth > 1) {
        out += 'array [\n';
		level = level + '    ';
        $.each (obj, function (key, val) {
            out += level + key + ' : ' + showObj (val, depth - 1) + '\n';
        });
        out += ']';
	} else if (type === 'object' && obj !== null && depth > 1) {
        out += 'object (' + obj.constructor.name + ') : {\n';
    	level = level + '    ';		
        $.each (obj, function (key, val) {
            out += level + key + ' : ' + showObj (val, depth - 1) + '\n';
        });
        out += '}';
    } else if (type === 'object' && obj !== null && depth <= 1) {
        out = level + '[object]\n';
    } else if (type === 'string' || type === 'number') {
		out = level + type + ': ' + obj;
	} else if (obj && typeof (obj.toString) !== 'undefined') {
        out = level + type + ': ' + obj.toString ();
    } else {
        out = level + 'null';
    }
    return out;
}
