<?php
// Credits:
$wgExtensionCredits ['parserhook'] [] = array (
	'path'			 => __FILE__
  , 'name'		 	 => 'Poll'
  ,	'descriptionmsg' => 'poll-desc'
  , 'version'		 => 0.1
  , 'author'		 => 'Alexander Mashin'
  , 'url'			 => 'http://traditio-ru.org/wiki/Poll_(расширение_MediaWiki)'
  , 'license-name'	 => ''
);
// Messages:
$wgExtensionMessagesFiles ['Poll'] = dirname(__FILE__) . '/Poll.i18n.php';
$wgExtensionMessagesFiles ['PollMagic'] = dirname(__FILE__) . '/Poll.i18n.magic.php';


// Settings:
$wgPollAPIpath = '/w/api.php';
$wgRatedNamespaces = array (NS_MAIN);
$wgPollRevote = true;
$wgPollShowResultsToNonVoters = true;

// Scripts:
$wgResourceModules ['ext.poll'] = array (
	'scripts' => 'Poll.js'
  ,	'styles' => array ('Poll.css')
  , 'messages' => array (
		'poll-loadfail', 'poll-sendfail'		
	  , 'poll-submit', 'poll-sent', 'poll-resubmit', 'poll-voted'
	  , 'poll-option', 'poll-option-readonly', 'poll-option-nostats'
	  , 'poll-loading'
	  , 'poll-option-color-0', 'poll-option-color-1', 'poll-option-color-2'
	  , 'poll-option-color-3', 'poll-option-color-4', 'poll-option-color-5'
	  , 'poll-rating-widget'
	  , 'poll-rating-1', 'poll-rating-2', 'poll-rating-3', 'poll-rating-4', 'poll-rating-5'
	  , 'poll-page-raters', 'poll-page-no-raters', 'poll-page-you-first'
	  , 'poll-rating-rated', 'poll-rating-not-rated', 'poll-rating-rerate', 'poll-rating-rate', 'poll-rating-rate-first'
	) // -- 'messages' => array (...)
// 	'dependencies' => array ()
  , 'localBasePath' => __DIR__
  , 'remoteExtPath' => 'Poll'
);
$wgHooks ['BeforePageDisplay'] [] = function (OutputPage &$out, Skin &$skin) {
	global $wgPollAPIpath, $wgPollRevote, $wgPollShowResultsToNonVoters;
	$revote	= var_export ($wgPollRevote, true);
	$show	= var_export ($wgPollShowResultsToNonVoters, true);
	$out->addScript (<<<EOT
<script language="JavaScript" type="text/javascript">
	var poll_api_path	= '$wgPollAPIpath';
	var poll_revote 	= $revote;
	var poll_show		= $show;
</script>
EOT
	);
	$out->addModules (array('ext.poll'));
};

$wgHooks ['BeforePageDisplay'] [] = function (OutputPage &$out, Skin &$skin) {
	global $wgRatedNamespaces;
	if (in_array ($skin->getTitle ()->getNamespace (), $wgRatedNamespaces)) {
		$out->addScript (<<<EOT
<script language="JavaScript" type="text/javascript">
$('#firstHeading').prepend ('<div class="ratingWidget"></div>');
</script>
EOT
		);
	}
	return true;
};


// Register parser functions:
$wgHooks ['ParserFirstCallInit'] [] = 'Poll::register';

// Register APIs:
$wgAPIModules ['poll'] = 'Poll';
$wgAPIModules ['rate'] = 'Rate';

// Register schema changes:
$wgHooks ['LoadExtensionSchemaUpdates'] [] = 'Poll::addTables';

class Poll extends ApiBase {

	// Database tables:
	const POLLTAB	  = 'polls';
	const OPTIONTAB   = 'poll_options';
	const ANSWERTAB   = 'poll_answers';
	const RATINGTAB	  = 'poll_ratings';	
	// Database columns:
	const POLLID	  = 'pollid';
	const POLLTEXT	  = 'polltext';	
	const POLLVOTES   = 'pollvotes';	
	const OPTIONID	  = 'optionid';
	const OPTIONTEXT  = 'optiontext';
	const OPTIONORDER = 'optionorder';
	const OPTIONVOTES = 'optionvotes';
	const USERID	  = 'userid';
	const USERNAME	  = 'username';
	
	// Extension permissions:
	const VOTE_PERMISSION  = 'vote';
	
	public static function register (Parser $parser) {
		// Register the functions:
		$parser->setFunctionHook ('poll-command', 'Poll::render');
		return true;
	}
	
	public static function render (Parser $parser) {
		// Getting the question and answer options:
		$no_params = func_num_args ();
		if ($no_params === 1) {
			return self::errorMsg ('poll-noquestion');
		} else if ($no_params < 4) {
			return self::errorMsg ('poll-fewoptions');
		}
		$options = func_get_args ();
		array_shift ($options); // -- first parametre is $parser.
		$question = array_shift ($options);
		$id = self::hash ($question);

		global $wgLang;
		$optionsHTML = '';
		foreach ($options as $option) {
			$optionsHTML .= wfMessage ('poll-option')->inLanguage ($wgLang->getCode ())->rawParams (
				'', '', '', $option, '', ''
			)->plain () . "\n";
		}
		$form = wfMessage ('poll-form')->inLanguage ($wgLang->getCode ())->rawParams (
			$id, $question, $optionsHTML
		)->plain () . "\n";
		
		// Register or alter the poll in the database:
		global $wgHooks;
		$wgHooks ['PageContentSaveComplete'] [] = array (__CLASS__ . '::savePoll', $id, $question, $options);
		
		return array ($form, 'noparse' => true, 'isHTML' => true);
	} // -- public static function render (Parser $parser)
		
	private static function errorMsg (/* string */ $msg) {
		return wfMessage ( 'poll-' . $msg)->inContentLanguage ()->plain ();
	}
	
	private static function hash (/* string */ $str) {
		return sha1 (preg_replace ('/[.,;:\'"«»!?()\s —-]/u', '', mb_convert_case (strip_tags ($str), MB_CASE_LOWER)));
	}
	
	// Load poll data:
	private static function loadPoll (/* string */ $pollid, /* string */ $userid, /* string */ $username) {
		$dbw = wfGetDB (DB_SLAVE);
		
		// Poll data:
		$res = $dbw->select (
			self::POLLTAB // -- table.
		  , array (self::POLLID, self::POLLTEXT, self::POLLVOTES) // -- columns.
		  , array (self::POLLID => $pollid)
		  , __METHOD__ // -- caller.
		);
		if ($res->numRows () === 0) {
			// A new poll:
			return null;
		}
		$row = $res->fetchRow ();
		$text      = $row [self::POLLTEXT];
		$pollvotes = $row [self::POLLVOTES];
		// Options:
		$options_res = $dbw->select (
			array (self::OPTIONTAB) // -- tables.
		  , array ( // -- columns.
				self::POLLID
			  , self::OPTIONID
			  , self::OPTIONTEXT
			  , self::OPTIONVOTES
			  , self::OPTIONORDER
			)
		  , array (self::POLLID => $pollid) // -- conditions.
		  , __METHOD__ // -- caller.
		  , array ('ORDER BY' => self::OPTIONORDER)
		); // -- $options_res = $dbw->select
		$options_no = $options_res->numRows ();
		$options = array ();
		for ($i = 0; $i < $options_no; $i++) {
			$option = $options_res->fetchRow ();
			$options [$option [self::OPTIONORDER]] = $option;
		}
		return array (
			self::POLLID 	=> $pollid
		  , self::POLLTEXT	=> $text
		  , self::POLLVOTES => $pollvotes
		  , 'options'		=> $options
		  , 'uservote'		=> self::getUserVote ($pollid, $userid, $username)			); // -- return array (...)
	} // -- private static function loadPoll (...)
	
	// Save poll to DB smartly:
	public static function savePoll (/* string */ $id, /* string */ $question, array $options) {

		// What have we got already?
		global $wgUser;
		$saved = self::loadPoll ($id, $wgUser->getID (), $wgUser->getName ());
		$dbw = wfGetDB (DB_MASTER);
		$dbw->begin (__METHOD__);
		if (!$saved) {
			// A new poll:
			$dbw->insert (
				self::POLLTAB // -- table.
			  , array (self::POLLID => $id, self::POLLTEXT => $question) // -- columns.
			  , __METHOD__
			);
		} else {
			// An existing poll that may be corrected:
			if ($question !== $saved [self::POLLTEXT]) {
				$dbw->update (
					self::POLLTAB // -- table.
				  , array (self::POLLTEXT . " = " . $dbw->addQuotes ($question)) // -- updated columns.
				  , array (self::POLLID => $id) // -- condition.
				  , __METHOD__
				);
			} // -- if ($question !== $saved ['text'])
			// Delete options with zero votes:
			$dbw->delete (
				self::OPTIONTAB // -- table.
			  , array (self::POLLID => $id, self::OPTIONVOTES => 0) // -- condition.
			  , __METHOD__
			);
		} // -- if (!$saved)
		// Insert new options and correct options already voted for:
		foreach ($options as $order => $option) {
			self::saveOption ($dbw, $id, $option, $order);
		} // -- foreach ($options as $option)
		$dbw->commit (__METHOD__);
		return true;
	} // -- private static function savePoll (/* string */ $id, /* string */ $question, array $options)
	
	private static function saveOption (/* DataBase */ $db, /* string */ $pollid, /* string */ $text, /* int */ $order) {
		$optid = self::hash ($text);
		$saved = $db->select (
			self::OPTIONTAB // -- table
		  , self::POLLID // -- column.
		  , array (self::POLLID => $pollid, self::OPTIONID => $optid) // -- conditions.
		  , __METHOD__ // -- caller.
		); // -- $res = $db->select (...)
		if ($saved->numRows () === 0) {
			// A new option:
			$db->insert (
				self::OPTIONTAB // -- table.
			  , array (
					self::POLLID 	  => $pollid
				  , self::OPTIONID 	  => $optid
				  , self::OPTIONORDER => $order
				  , self::OPTIONTEXT  => $text
				) // -- columns.
			  , __METHOD__
			); // -- $db->insert (...)
		} else {
			// An existing option:
			$db->update (
				self::OPTIONTAB // -- table.
			  , array (self::OPTIONTEXT . " = " . $db->addQuotes ($text)) // -- updated columns.
			  , array (self::POLLID => $pollid, self::OPTIONID => $optid) // -- condition.
			  , __METHOD__
			); // -- $db->update (...)
		} // -- if (count ($saved) === 0)
	} // -- private static function insertOption (DataBase $db, /* string */ $pollid, /* string */ $text)
	
	private static function getUserVote (/* string */ $pollid, /* string */ $userid, /* string */ $username) {
		$dbw = wfGetDB (DB_SLAVE);
		$saved = $dbw->select (
			self::ANSWERTAB // -- table
		  , self::OPTIONID // -- column.
		  , array ( // -- conditions.
				self::POLLID   => $pollid
			  , self::USERID   => $userid
			  , self::USERNAME => $username
			)
		  , __METHOD__ // -- caller.
		); // -- $res = $dbw->select (...)
		return $saved->numRows () === 0 ? '' : $saved->fetchRow () [self::OPTIONID];
	} // -- private static function getUserVote (/* string */ $id, /* string */ $userid, /* string */ $username)
	
	private static function saveAnswer (
		/* int */	 $pollid
	  , /* int */	 $userid
	  , /* string */ $username
	  , /* string */ $optionid
	) {
		$dbw = wfGetDB (DB_MASTER);
		$former_option = self::getUserVote ($pollid, $userid, $username);
		$dbw->begin (__METHOD__);
		if ($former_option) {
			// Changed my mind:
			$dbw->delete (
				self::ANSWERTAB // -- table.
			  , array (
					self::POLLID   => $pollid
				  , self::USERID   => $userid
				  , self::USERNAME => $username
				) // -- condition.
			  , __METHOD__
			); // -- $dbw->delete (...)
			if ($former_option !== $optionid) {
				$dbw->update (
					self::OPTIONTAB // -- table.
				  , array (self::OPTIONVOTES . ' = ' . self::OPTIONVOTES . ' -1') // -- updated columns.
				  , array (self::POLLID => $pollid, self::OPTIONID => $former_option) // -- condition.
				  , __METHOD__
					); // -- $db->update (...)
			} // -- if ($former_option !== $optionid)
		} // -- if ($vote)
		$dbw->insert (
			self::ANSWERTAB // -- table.
		  , array (
				self::POLLID   => $pollid
			  , self::USERID   => $userid
			  , self::USERNAME => $username
			  , self::OPTIONID => $optionid
			) // -- columns.
		  , __METHOD__
		); // -- $dbw->insert (...)

		// Update counts:
		if (!$former_option) {
			$dbw->update (
				self::POLLTAB // -- table.
			  , array (self::POLLVOTES . ' = ' . self::POLLVOTES . ' + 1') // -- updated columns.
			  , array (self::POLLID => $pollid) // -- condition.
			  , __METHOD__
			); // -- $db->update (...)
		} // -- if (!$vote)
		if ($former_option !== $optionid) {
			$dbw->update (
				self::OPTIONTAB // -- table.
			  , array (self::OPTIONVOTES . ' = ' . self::OPTIONVOTES . ' + 1') // -- updated columns.
			  , array (self::POLLID => $pollid, self::OPTIONID => $optionid) // -- condition.
			  , __METHOD__
			); // -- $db->update (...)
		} // -- if ($former_option !== $optionid)
		$dbw->commit (__METHOD__);
	} // -- private static function saveAnswer (...)
	
	// Called by API from JavaScript (Poll.js):
	public function execute () {
	
		global $wgUser;
		$can_vote = $wgUser->isAllowed (self::VOTE_PERMISSION);

		$params = $this->extractRequestParams ();
		// Voting:
		if ($params ['option'] && $can_vote) {
			self::saveAnswer ($params ['poll'], $wgUser->getID (), $wgUser->getName (), $params ['option']);
		}
		$this->getResult ()->addValue (null, 'canvote', $can_vote ? 1 : 0);
		// Display results:
		$poll = self::loadPoll ($params ['poll'], $wgUser->getID (), $wgUser->getName ());

		foreach ($poll as $key => $val) {
			if ($key === 'options') {
				//$this->getResult ()->setIndexedTagName ($val, 'options');
				foreach ($val as $order => $option) {
					wfDebug (var_export ($option, true) . "\n");
					$this->getResult ()->addValue (array ('options', $order), self::OPTIONID   , $option [self::OPTIONID]);
					$this->getResult ()->addValue (array ('options', $order), self::OPTIONTEXT , $option [self::OPTIONTEXT]);
					$this->getResult ()->addValue (array ('options', $order), self::OPTIONVOTES, $option [self::OPTIONVOTES]);
				}
			} else {
				$this->getResult ()->addValue (null, $key, $val);
			} // -- if ($key === 'options')
		} // -- foreach ($poll as $key => $val)
		return true;
	} // -- public function execute ()
	
	// Parametres for api.php?action=poll:
	public function getAllowedParams () {
		return array (
			'poll' => array (
				ApiBase::PARAM_TYPE => 'string',
				ApiBase::PARAM_REQUIRED => true
			)
		  , 'option' => array (
				ApiBase::PARAM_TYPE => 'string',
				ApiBase::PARAM_REQUIRED => false
			)
		  , '_' => array (
				ApiBase::PARAM_TYPE => 'string',
				ApiBase::PARAM_REQUIRED => false
			)
		);
	} // -- public function getAllowedParams ()

	protected function getParamDescription () {
		global $wgLang;
		$params = parent::getParamDescription ();
		$params = is_array ($params) ? $params : array ($params);
		return array_merge ($params, array (
			'poll'   =>  wfMessage ('poll-api-poll-desc'  )->inLanguage ($wgLang->getCode ())->parse ()
		  , 'option' =>  wfMessage ('poll-api-option-desc')->inLanguage ($wgLang->getCode ())->parse ()
		));
	} // -- public static function getParamDescription (&$module, &$desc)
	
	protected function getDescription () {
		global $wgLang;
		return wfMessage ('poll-api-desc')->inLanguage ($wgLang->getCode ())->parse ();
	}

	public static function addTables ($updater) {
		$dir = dirname ( __FILE__ );
		$file = "$dir/poll.sql";

		$updater->addExtensionUpdate (array ('addTable', self::POLLTAB  , $file, true));
		$updater->addExtensionUpdate (array ('addTable', self::OPTIONTAB, $file, true));
		$updater->addExtensionUpdate (array ('addTable', self::ANSWERTAB, $file, true));
		return true;
	}
} // -- class Poll

		
class Rate extends ApiBase {

	// Database tables:

	const RATINGTAB	  = 'poll_ratings';	
	const RATERTAB	  = 'poll_raters';		
	// Database columns:
	const USERID	  = 'userid';
	const USERNAME	  = 'username';
	const PAGEID	  = 'pageid';
	const PAGERATING  = 'pagerating';
	const USERRATING  = 'userrating';
	const PAGEVOTES   = 'pagevotes';		
	
	// Extension permissions:
	const RATE_PERMISSION  = 'rate';
	
	private static function errorMsg (/* string */ $msg) {
		return wfMessage ( 'poll-' . $msg)->inContentLanguage ()->plain ();
	}
	
	// Load page rating:
	private static function loadRating (/* int */ $pageid, /* string */ $userid, /* string */ $username) {
		$dbw = wfGetDB (DB_SLAVE);
		
		// Load total votes:
		$res = $dbw->select (
			self::RATINGTAB // -- table.
		  , array (self::PAGERATING, self::PAGEVOTES) // -- columns.
		  , array (self::PAGEID => $pageid) // -- conditions.
		  , __METHOD__ // -- caller.
		);
		$total_rating = 0;
		$votes		  = 0;
		$rating		  = 0;
		if ($res->numRows () !== 0) {
			$row = $res->fetchRow ();
			$total_rating = $row [self::PAGERATING];
			$votes 		  = $row [self::PAGEVOTES];
			
			// Load current user's page rating:
			$res = $dbw->select (
				self::RATERTAB // -- table.
			  , self::PAGERATING // -- columns.
			  , array ( // -- conditions.
					self::PAGEID   => $pageid
				  , self::USERID   => $userid
				  , self::USERNAME => $username
				)
			  , __METHOD__ // -- caller.
			); // -- $res = $dbw->select (...)
			if ($res->numRows () !== 0) {
				$rating = $res->fetchRow () [self::PAGERATING];
			}
		} // -- if ($res->numRows () !== 0)
			
		return array (
			self::PAGEID 	 => $pageid
		  , self::USERID 	 => $userid
		  , self::USERNAME   => $username
		  , self::PAGERATING => $total_rating
		  , self::PAGEVOTES  => $votes
		  , self::USERRATING => $rating
		); // -- return array (...)
	} // -- private static function loadRating (...)
	
	// Save page rating to DB smartly:
	public static function saveRating (
		/* int */    $pageid
	  , /* int */    $userid
	  , /* string */ $username
	  , /* int */    $rating
	) {
		// What have we got already?
		$saved = self::loadRating ($pageid, $userid, $username);
		$dbw = wfGetDB (DB_MASTER);
		$dbw->begin (__METHOD__);
		$old_rating = $saved [self::USERRATING];
		if (!$old_rating) {
			// A page previously unrated by the current user:
			$dbw->insert (
				self::RATERTAB // -- table.
			  , array (
					self::PAGEID     => $pageid
				  , self::USERID     => $userid
				  , self::USERNAME   => $username
				  , self::PAGERATING => $rating) // -- columns.
			  , __METHOD__
			); // -- $dbw->insert (...)
		} else {
			// Update page user's rating:
			$dbw->update (
				self::RATERTAB // -- table.
			  , array (self::PAGERATING . ' = ' . self::PAGERATING . ' + ' . ($rating - $old_rating)) // -- updated columns.
			  , array ( // -- condition.
					self::PAGEID   => $pageid
				  , self::USERID   => $userid
				  , self::USERNAME => $username
				) 
			  , __METHOD__
			); // -- $dbw->update (...)
		} // -- if (!$saved)

		// Update page's number of raters:
		if ($saved [self::PAGEVOTES]) {
			if (!$old_rating) {
				// This user has not rate this page yet. So update tge statictics.

				$dbw->update (
					self::RATINGTAB // -- table.
				  , array ( // -- updated columns.
						self::PAGEVOTES . ' = ' . self::PAGEVOTES . ' + 1'
					)
				  , array (self::PAGEID => $pageid) // -- condition.
				  , __METHOD__
				); // -- $dbw->update (...)
			} // -- if (!$old_rating)
			$dbw->update (
				self::RATINGTAB // -- table.
			  , array ( // -- updated columns.
					self::PAGERATING . ' = ' . self::PAGERATING . ' + ' . ($rating - $old_rating)
				)
			  , array (self::PAGEID => $pageid) // -- condition.
			  , __METHOD__
			); // -- $dbw->update (...)
		} else {
			// A page previously unrated by anyone:
			$dbw->insert (
				self::RATINGTAB // -- table.
			  , array (
					self::PAGEID     => $pageid
				  , self::PAGEVOTES	 => 1
				  , self::PAGERATING => $rating) // -- columns.
			  , __METHOD__
			); // -- $dbw->insert (...)
		} // -- if ($saved [self::PAGEVOTES])
		
		$dbw->commit (__METHOD__);
		return true;
	} // -- public static function saveRating (...)
	
	// Called by API from JavaScript (Poll.js):
	public function execute () {
	
		global $wgUser;
		$can_rate = $wgUser->isAllowed (self::RATE_PERMISSION);

		$params = $this->extractRequestParams ();
		// Rating:
		if ($can_rate && $params ['rating'] && $params ['rating'] >= 1 && $params ['rating'] <= 5) {
			self::saveRating ($params ['pageid'], $wgUser->getID (), $wgUser->getName (), $params ['rating']);
		}
		
		$this->getResult ()->addValue (null, 'canrate', $can_rate ? 1 : 0);
		
		// Display results:
		$rating = self::loadRating ($params ['pageid'], $wgUser->getID (), $wgUser->getName ());

		foreach ($rating as $key => $val) {
			$this->getResult ()->addValue (null, $key, $val);
		} // -- foreach ($rating as $key => $val)
		return true;
	} // -- public function execute ()
	
	// Parametres for api.php?action=poll:
	public function getAllowedParams () {
		return array (
			'pageid' => array (
				ApiBase::PARAM_TYPE => 'integer',
				ApiBase::PARAM_REQUIRED => true
			)
		  , 'rating' => array (
				ApiBase::PARAM_TYPE => 'integer',
				ApiBase::PARAM_REQUIRED => false
			)
		  , '_' => array (
				ApiBase::PARAM_TYPE => 'string',
				ApiBase::PARAM_REQUIRED => false
			)
		);
	} // -- public function getAllowedParams ()

	protected function getParamDescription () {
		global $wgLang;
		$params = parent::getParamDescription ();
		$params = is_array ($params) ? $params : array ($params);
		return array_merge ($params, array (
			'pageid' =>  wfMessage ('poll-api-rate-desc'  )->inLanguage ($wgLang->getCode ())->parse ()
		  , 'rating' =>  wfMessage ('poll-api-rating-desc')->inLanguage ($wgLang->getCode ())->parse ()
		));
	} // -- public static function getParamDescription (&$module, &$desc)
	
	protected function getDescription () {
		global $wgLang;
		return wfMessage ('poll-api-desc')->inLanguage ($wgLang->getCode ())->parse ();
	}

	public static function addTables ($updater) {
		$dir = dirname ( __FILE__ );
		$file = "$dir/poll.sql";
		$updater->addExtensionUpdate (array ('addTable', self::RATINGTAB, $file, true));
		$updater->addExtensionUpdate (array ('addTable', self::RATERTAB, $file, true));		
		return true;
	}
						  
} // -- class Rate
		
		
// Unfortunately, I need my own API format:
$wgAPIFormatModules ['plain'] = 'PlainParseAPI';
class PlainParseAPI extends ApiFormatBase {
	public function execute () {
		$this->printText ($this->getResultData () ['parse'] ['text']);
	}
	public function getMimeType () {
		return 'application/json';
	}
} // -- class PlainParseAPI extends ApiFormatBase


// Permissions:
$wgAvailableRights [] = Poll::VOTE_PERMISSION;
$wgGroupPermissions ['*'] [Poll::VOTE_PERMISSION] = true;

$wgAvailableRights [] = Rate::RATE_PERMISSION;
$wgGroupPermissions ['user'] [Rate::RATE_PERMISSION] = true;

?>
